//-----------------------------------------------------------------------------
// Torque Game Engine 
// Copyright (C) GarageGames.com, Inc.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// AIPlayer callbacks
// The AIPlayer class implements the following callbacks:
//
//    PlayerData::onStuck(%this,%obj)
//    PlayerData::onUnStuck(%this,%obj)
//    PlayerData::onStop(%this,%obj)
//    PlayerData::onMove(%this,%obj)
//    PlayerData::onReachDestination(%this,%obj)
//    PlayerData::onTargetEnterLOS(%this,%obj)
//    PlayerData::onTargetExitLOS(%this,%obj)
//    PlayerData::onAdd(%this,%obj)
//
// Since the AIPlayer doesn't implement it's own datablock, these callbacks
// all take place in the PlayerData namespace.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Demo Pathed AIPlayer.
//-----------------------------------------------------------------------------
exec("~/data/shapes/guard_player/player.cs");
datablock PlayerData(DemoPlayer : PlayerBody)
{
	   shapeFile = "~/data/shapes/guard_player/player.dts";
   maxInv[CrossbowAmmo] = 1000;
   shootingDelay = 2000;
   maxForwardSpeed = 6;
   maxBackwardSpeed = 4;
   maxSideSpeed = 4;
  path = "";
  index = "";
  Sstate = "Guard";
};

function DemoPlayer::onReachDestination(%this,%obj)
{
   // Moves to the next node on the path.
   // Override for all player. Normally we'd override this for only
   // a specific player datablock or class of players.
   if (%obj.path !$= "") {
      if (%obj.currentNode == %obj.targetNode)
         %this.onEndOfPath(%obj,%obj.path);
      else
         %obj.moveToNextNode();
   }
}

function DemoPlayer::onEndOfPath(%this,%obj,%path)
{
   %obj.nextTask();
}

function DemoPlayer::onEndSequence(%this,%obj,%slot)
{
   //echo("Sequence Done!");
   %obj.stopThread(%slot);
   %obj.nextTask();
}


//-----------------------------------------------------------------------------
// AIPlayer static functions
//-----------------------------------------------------------------------------

function AIPlayer::spawn(%name,%spawnPoint)
{
   // Create the demo player object
   %player = new AiPlayer() {
      dataBlock = DemoPlayer;

   };
   MissionCleanup.add(%player);
   %player.setShapeName(%name);
   %player.setTransform(%spawnPoint);
   return %player;
}

function AIPlayer::spawnOnPath(%name,%path)
{
   // Spawn a player and place him on the first node of the path
   if (!isObject(%path))
      return;
   %node = %path.getObject(0);
   %player = AIPlayer::spawn(%name,%node.getTransform());
   return %player;
}


//-----------------------------------------------------------------------------
// AIPlayer methods 
//-----------------------------------------------------------------------------

function AIPlayer::followPath(%this,%path,%node)
{
   // Start the player following a path
	//echo("Follow path: " @%path);
   %this.stopThread(0);
   if (!isObject(%path)) {
      %this.path = "";
      return;
   }
   if (%node > %path.getCount() - 1)
      %this.targetNode = %path.getCount() - 1;
   else
      %this.targetNode = %node;
   if (%this.path $= %path)
      %this.moveToNode(%this.currentNode);
   else {
      %this.path = %path;
      %this.moveToNode(0);
   }
}

function AIPlayer::moveToNextNode(%this)
{
   if (%this.targetNode < 0 || %this.currentNode < %this.targetNode) {
      if (%this.currentNode < %this.path.getCount() - 1)
         %this.moveToNode(%this.currentNode + 1);
      else
         %this.moveToNode(0);
   }
   else
      if (%this.currentNode == 0)
         %this.moveToNode(%this.path.getCount() - 1);
      else
         %this.moveToNode(%this.currentNode - 1);
}

function AIPlayer::moveToNode(%this,%index)
{
   // Move to the given path node index
   %this.currentNode = %index;
   %node = %this.path.getObject(%index);
   %this.setMoveDestination(%node.getTransform(), %index == %this.targetNode);
}


//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

function AIPlayer::pushTask(%this,%method)
{
   if (%this.taskIndex $= "") {
      %this.taskIndex = 0;
      %this.taskCurrent = -1;
   }
   %this.task[%this.taskIndex] = %method; 
   %this.taskIndex++;
   if (%this.taskCurrent == -1)
      %this.executeTask(%this.taskIndex - 1);
}

function AIPlayer::clearTasks(%this)
{
   %this.taskIndex = 0;
   %this.taskCurrent = -1;
}

function AIPlayer::nextTask(%this)
{
   if (%this.taskCurrent != -1)
      if (%this.taskCurrent < %this.taskIndex - 1)
         %this.executeTask(%this.taskCurrent++);
      else
         %this.taskCurrent = -1;
}

function AIPlayer::executeTask(%this,%index)
{
   %this.taskCurrent = %index;
   eval(%this.getId() @ "." @ %this.task[%index] @ ";");
}


//-----------------------------------------------------------------------------

function AIPlayer::singleShot(%this)
{
   // The shooting delay is used to pulse the trigger
   %this.setImageTrigger(0,true);
   %this.setImageTrigger(0,false);
   %this.trigger = %this.schedule(%this.shootingDelay,singleShot);
}


//-----------------------------------------------------------------------------

function AIPlayer::wait(%this,%time)
{
   %this.schedule(%time * 1000,"nextTask");
}

function AIPlayer::done(%this,%time)
{
   %this.schedule(0,"delete");
}

function AIPlayer::fire(%this,%bool)
{
   if (%bool) {
      cancel(%this.trigger);
      %this.singleShot();
   }
   else
      cancel(%this.trigger);
   %this.nextTask();
}

function AIPlayer::aimAt(%this,%object)
{
   //echo("Aim: " @ %object);
   %this.setAimObject(%object);
   %this.nextTask();
}

function AIPlayer::animate(%this,%seq)
{
   //%this.stopThread(0);
   //%this.playThread(0,%seq);
   %this.setActionThread(%seq);
}


//-----------------------------------------------------------------------------

function DemoPlayer::think(%this,%obj)
{
   // We could hook into the player's onDestroyed state instead of
   // having to "think", but thinking allows us to consider other
   // things...
   //if (!isObject(%this.player))
      //%this.player = %this.spawn();

	if (!isObject(%obj))
		return;
	if (%obj.getState() $= "Dead")
		return;
	
	%count = clientGroup.getCount();
	if (%count < 1)
		%client = -1;
	else
		%client = clientGroup.getObject(0);
	
	
	//echo("Client State:" @%client.player.getState());
	if (%client.player.getState() $= "")
	{
		%obj.setImageTrigger(0,false);
	   %this.schedule(500,"think",%obj);
		return;
	}
	
	//if (%obj.Sstate = "Guard")
	
		if (%client != -1)
		{
			%playerLoc = %client.player.getPosition();
			%robotLoc = %obj.getWorldBoxCenter();
			%dist = VectorDist(%robotLoc,%playerLoc);
			
		if (%dist < 30)
			{
				%obj.setAimObject( %client, "0 0 1" );
				%obj.setAimLocation(VectorAdd(%playerLoc,"0 0 1"));
				%obj.setImageTrigger(0,true);
			}
			if (%dist < 40 && %dist > 30)
			{
				%obj.setAimObject( %client, "0 0 1" );
				%obj.setAimLocation(VectorAdd(%playerLoc,"0 0 1"));
				%obj.setImageTrigger(0,true);
				%obj.setMoveDestination(%client.player.getPosition());
			}
			
			if (%dist >40 && %dist < 50)
			{
				%obj.setImageTrigger(0,false);
				%obj.setMoveDestination(%client.player.getPosition());
				//%obj.Sstate = "Running";
			}
			
			if (%dist > 50)
			{
				%obj.Sstate = "Guard";
				%obj.setImageTrigger(0,false);
				%obj.setMoveDestination(%robotLoc);
				%obj.followPath(%obj.path,-1);
//echo("path :" @%obj.path);
			}
		}
	
//echo("%client:" @ %client.player);
//echo("%distance:" @ %dist);
//echo("%this:" @ %obj);
   
   %this.schedule(500,"think",%obj);
}

function createRobots()
{
	%group = nameToID("MissionGroup/AIDropPoints/Guard");
	%count = %group.getCount();
	//echo("The count:" @%count);
	for(%i=0;%i<%count;%i++)
	{
		spawnGuards(%i,"Guard");
	}
}

function spawnGuards(%index,%role)
{	
	//echo("The index is:" @%index);
	%me = new AIPlayer(){
		datablock = DemoPlayer;
		aiPlayer = true;
		Sstate = "Guard";
		maxForwardSpeed = 10;
	};
	MissionCleanup.add(%me);
	AIGroup.add(%me);
	%spawn = pickAISpawn (%index,%role);
	%me.setTransform(%spawn.getTransform());
	%me.path = %spawn.fpath;
	//echo("Robot Path: "@%me.path);
	%me.equip(Crossbow2,Crossbow2Ammo);
	//%me.followPath(%me.path,-1);
	%me.getdatablock().schedule(500,"think",%me);
	return %me;
}

function pickAISpawn(%index,%role)
{
	//echo("The Role is:" @%role);
   %groupName = "MissionGroup/AIDropPoints/" @%role;
   %group = nameToID(%groupName);

   if (%group != -1) 
   {
      %count = %group.getCount();
      if (%count != 0) 
      {
        // %index = getRandom(%count-1);
         %spawn = %group.getObject(%index);
         return %spawn;
      }
      else
            echo("No spawn points found in " @ %groupName, "pickAISpawn");
   }
   else
       echo("Missing spawn points group " @ %groupName, "pickAISpawn");

   return %spawn;
}

function AIPlayer::Equip(%this,%weaponDBName,%ammoDBName)
{
   //echo( "%this:"@%this@"~AIPlayer::Equip", Equip);
   %weapon = new Item() {
      dataBlock = %weaponDBName;
   };
   %ammo = new Item() {
      dataBlock = %ammoDBName;
   };
   //echo("weapon:"@%weaponDBName, "Equip");
   //echo("ammo:"@%ammoDBName, "Equip");
   MissionCleanup.add(%weapon);
   MissionCleanup.add(%ammo);
   %weaponImageName = %weaponDBName @"Image";
  %this.mountImage(%weaponImageName,0);
  %this.setInventory(%ammoDBName,1000);
  %this.use(%weaponDBName);
}






